function createColumn(dataType, name, filter, visible, renderFunction) {
    if (renderFunction === undefined) {
        renderFunction = function (data, type, row, meta) {
            return row[dataType];
        }
    }
    return {
        "type": dataType,
        "name": name,
        "filter": filter,
        "visible": visible,
        "render": renderFunction
    };
}

function getColumns(type) {
    var result = [];
    // don't confuse end user
    // result.push(createColumn("id", "Id", null, false));
    result.push(createColumn("nd_number", "ND", "string_filter", true));
    result.push(createColumn("screening_number", "Screening", "string_filter", true));
    result.push(createColumn("first_name", "First name", "string_filter", true));
    result.push(createColumn("last_name", "Last name", "string_filter", true));
    result.push(createColumn("date_born", "Date of birth", null, false));
    result.push(createColumn("default_location", "Location", "location_filter", true));
    result.push(createColumn("dead", "Deceased", "yes_no_filter", true));
    result.push(createColumn("resigned", "Resigned", "yes_no_filter", true));
    result.push(createColumn("postponed", "Postponed", "yes_no_filter", true));
    result.push(createColumn("information_sent", "Info sent", "yes_no_filter", true));
    result.push(createColumn("type", "Type", "type_filter", true));
    result.push(createColumn("id", "Edit", null, true, function (data, type, row, meta) {
        return '<a href="#" type="button" class="btn btn-block btn-default">Edit</a>';
    }));
    for (var i = 1; i <= 8; i++) {
        var renderFunction = (function () {
            var x = i;
            return function (data, type, row, meta) {
                return create_visit_row(row.visits[x - 1]);
            };
        })();

        result.push(createColumn("visit_" + i, "Visit " + i, "visit_filter", true, renderFunction));

    }
    return result;

}

function createHeader(columnsDefinition) {
    var header = document.createElement("thead");
    var headerRow = document.createElement("tr");
    header.appendChild(headerRow);
    for (var i = 0; i < columnsDefinition.length; i++) {
        var column = columnsDefinition[i];
        var element = document.createElement("th");
        element.innerHTML = column.name;
        headerRow.appendChild(element);
    }
    return header;
}

function createFilter(columnsDefinition) {
    var footer = document.createElement("tfoot");
    footer.style.display = "table-header-group";
    var footerRow = document.createElement("tr");
    footer.appendChild(footerRow);
    for (var i = 0; i < columnsDefinition.length; i++) {
        var column = columnsDefinition[i];
        var element = document.createElement("th");
        if (column.filter !== null) {
            element.innerHTML = "<div name='" + column.filter + "'>" + column.name + "</div>";
        }
        footerRow.appendChild(element);
    }
    return footer;
}

function create_visit_row(visit) {
    var color = "white";
    var text = "---";
    if (visit !== undefined && visit !== null) {
        if (visit.status === "DONE") {
            color = "green";
            text = "OK";
        } else if (visit.status === "MISSED") {
            color = "pink";
            text = "MISSED";
        } else if (visit.status === "UPCOMING") {
            color = "#00ffff";
            text = "UPCOMING";
        } else if (visit.status === "EXCEEDED") {
            color = "orange";
            text = "EXCEEDED";
        } else if (visit.status === "SHOULD_BE_IN_PROGRESS") {
            color = "orange";
            text = "IN PROGRESS (NO APPOINTMENTS)";
        } else if (visit.status === "IN_PROGRESS") {
            color = "lightgreen";
            text = "IN PROGRESS";
        }
        text += "<br/>" + visit.datetime_start + " - " + visit.datetime_end;
    }
    return "<div style='background-color:" + color + "';width:100%;height:100%>" + text + "</div>";
}

function createVisibilityCheckboxes(checkboxesElement, columns) {
    var row = null;
    for (var i = 0; i < columns.length; i++) {
        if (i % 10 === 0) {
            row = document.createElement("div");
            row.style.display = "table-row";
            checkboxesElement.appendChild(row);
        }
        var column = columns[i];
        var element = document.createElement("div");
        element.style.display = "table-cell";
        var checked = "";
        if (column.visible) {
            checked = "checked";
        }
        element.innerHTML = "<input type='checkbox' " + checked + " data-column='" + i + "' name='" + column.type + "'/>" + column.name;
        row.appendChild(element);
    }

}

function createSubjectsTable(params) {
    var tableElement = params.tableElement;
    var worker_locations = params.worker_locations;
    var getSubjectEditUrl = params.getSubjectEditUrl;
    var subject_types_url = params.subject_types_url;
    var locations_url = params.locations_url;
    var subjects_url = params.subjects_url;
    var columnsDefinition = params.columns;

    tableElement.appendChild(createHeader(columnsDefinition));
    tableElement.appendChild(createFilter(columnsDefinition));
    tableElement.appendChild(document.createElement("tbody"));
    createVisibilityCheckboxes(params.checkboxesElement, columnsDefinition);

    var table;
    $(tableElement).find('tfoot div[name="string_filter"]').each(function () {
        var title = $(this).text();
        $(this).html('<input type="text" style="width:80px" placeholder="' + title + '" />');
    });

    $(tableElement).find('tfoot div[name="yes_no_filter"]').each(function () {
        $(this).html('<select style="width:60px" ><option value selected="selected">---</option><option value="true">YES</option><option value="false">NO</option></select>');
    });

    $(tableElement).find('tfoot div[name="visit_filter"]').each(function () {
        $(this).html('<select style="width:60px" >' +
            '<option value selected="selected">---</option>' +
            '<option value="MISSED">MISSED</option>' +
            '<option value="DONE">DONE</option>' +
            '<option value="EXCEED">EXCEED</option>' +
            '<option value="IN_PROGRESS">IN PROGRESS</option>' +
            '<option value="SHOULD_BE_IN_PROGRESS">SHOULD BE IN PROGRESS</option>' +
            '<option value="UPCOMING">UPCOMING</option>' +
            '</select>');
    });

    $(tableElement).find('tfoot div[name="location_filter"]').each(function () {
        var obj = $(this);
        obj.html('<select style="width:80px"><option value selected="selected">---</option></select>');
        var select = $('select', obj);
        $.get(locations_url, function (data) {
            $.each(data.locations, function (index, location) {
                select.append('<option value="' + location.id + '">' + location.name + '</option>');
            });
            if (worker_locations.length === 1) {
                select.val(worker_locations[0].id);
            }
        });

    });
    $(tableElement).find('tfoot div[name="type_filter"]').each(function () {
        var obj = $(this);
        obj.html('<select style="width:80px"><option value selected="selected">---</option></select>');
        var select = $('select', obj);
        $.get(subject_types_url, function (data) {
            $.each(data.types, function (index, type) {
                select.append('<option value="' + type.id + '">' + type.name + '</option>');
            });
        });
    });


    $(function () {
        var columns = [];
        var columnDefs = [];
        for (var i = 0; i < columnsDefinition.length; i++) {
            var column = columnsDefinition[i];
            columns.push({"data": column.type});
            columnDefs.push({"targets": i, "render": column.render, visible: column.visible});
        }


        table = $('#table').DataTable({
            pageLength: 25,
            serverSide: true,
            processing: true,
            responsive: true,
            ajax: subjects_url,
            columns: columns,
            columnDefs: columnDefs,
            order: [[0, 'desc']]
        });

        $('#table tbody').on('click', 'a', function () {
            var data = table.row($(this).parents('tr')).data();
            window.location.href = getSubjectEditUrl(data.id.toString());
        });

        // Apply the search
        table.columns().every(function () {
            var that = this;

            $('input', this.footer()).on('keyup change', function () {
                if (that.search() !== this.value) {
                    that.search(this.value).draw();
                }
            });
            $('select', this.footer()).on('keyup change', function () {
                if (that.search() !== this.value) {
                    that.search(this.value).draw();
                }
            });
        });
        $('#table_filter').css("display", "null");
    });
    $('#visible-column-checkboxes input').on('click', function (e) {
        var visible = $(this).is(":checked");

        // Get the column API object
        var column = table.column($(this).attr('data-column'));
        console.log($(this).attr('data-column'));
        // Toggle the visibility
        column.visible(visible);
    });
}